<?php
/**
 * @package     com_easyshop
 * @version     1.2.5
 * @Author      JoomTech Team
 * @copyright   Copyright (C) 2015 - 2019 www.joomtech.net All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

defined('_JEXEC') or die;

use ES\Factory;
use Joomla\Utilities\ArrayHelper;
use ES\Helper\Navbar;
use Joomla\Registry\Registry;
use Joomla\CMS\Factory as CMSFactory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Component\ComponentHelper;

JLoader::import('easyshop.factory');

class PlgSystemEasyshop extends CMSPlugin
{
	/**
	 * @var  $app \JApplicationCms
	 * @since 1.0.0
	 */
	protected $app;

	public function onAfterInitialise()
	{
		self::defines();
		easyshop('addIncludeClassPath', ES_LIBRARIES . '/classes');
		HTMLHelper::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/helpers/html');

		// For Joomla! 3.9 privacy
		if (version_compare(JVERSION, '3.9', 'ge')
			&& ComponentHelper::isEnabled('com_privacy')
		)
		{
			easyshop('class.privacy', $this->_subject);
		}
	}

	public static function defines()
	{
		if (!defined('ES_VERSION'))
		{
			define('ES_VERSION', '1.2.5');
		}

		if (!defined('ES_VERSION_HASH'))
		{
			define('ES_VERSION_HASH', md5(ES_VERSION));
		}

		if (!defined('ES_DETECT_JVERSION'))
		{
			if (version_compare(JVERSION, '4.0', 'ge'))
			{
				$jVersion = 4;
			}
			else
			{
				$jVersion = 3;
			}

			define('ES_DETECT_JVERSION', $jVersion);
		}

		if (!defined('ES_COMPONENT'))
		{
			define('ES_COMPONENT', JPATH_BASE . '/components/com_easyshop');
		}

		if (!defined('ES_COMPONENT_SITE'))
		{
			define('ES_COMPONENT_SITE', JPATH_SITE . '/components/com_easyshop');
		}

		if (!defined('ES_COMPONENT_ADMINISTRATOR'))
		{
			define('ES_COMPONENT_ADMINISTRATOR', JPATH_ADMINISTRATOR . '/components/com_easyshop');
		}

		if (!defined('ES_MEDIA'))
		{
			define('ES_MEDIA', JPATH_ROOT . '/media/com_easyshop');
		}

		if (!defined('ES_MEDIA_URL'))
		{
			define('ES_MEDIA_URL', JUri::root(true) . '/media/com_easyshop');
		}

		if (!defined('ES_LIBRARIES'))
		{
			define('ES_LIBRARIES', JPATH_LIBRARIES . '/easyshop');
		}

		if (!defined('ES_ORDER_CREATED'))
		{
			define('ES_ORDER_CREATED', 0);
		}

		if (!defined('ES_ORDER_CONFIRMED'))
		{
			define('ES_ORDER_CONFIRMED', 1);
		}

		if (!defined('ES_ORDER_PROCESSED'))
		{
			define('ES_ORDER_PROCESSED', 2);
		}

		if (!defined('ES_ORDER_SHIPPED'))
		{
			define('ES_ORDER_SHIPPED', 3);
		}

		if (!defined('ES_ORDER_SUCCEED'))
		{
			define('ES_ORDER_SUCCEED', 4);
		}

		if (!defined('ES_ORDER_CANCELLED'))
		{
			define('ES_ORDER_CANCELLED', 5);
		}

		if (!defined('ES_ORDER_ORDER_TRASHED'))
		{
			define('ES_ORDER_ORDER_TRASHED', -2);
		}
	}

	public function onAfterRoute()
	{
		try
		{
			if (!JComponentHelper::isEnabled('com_easyshop'))
			{
				$this->_subject->detach($this);
				$db    = CMSFactory::getDbo();
				$query = $db->getQuery(true)
					->update($db->quoteName('#__extensions'))
					->set($db->quoteName('enabled') . ' = 0')
					->set($db->quoteName('protected') . ' = 0')
					->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
					->where($db->quoteName('folder') . ' = ' . $db->quote('system'))
					->where($db->quoteName('element') . ' = ' . $db->quote('easyshop'));

				return $db->setQuery($query)->execute();
			}

			$language = CMSFactory::getLanguage();
			$language->load('com_easyshop', ES_COMPONENT_ADMINISTRATOR, null, false, true)
			|| $language->load('com_easyshop', JPATH_ADMINISTRATOR, null, false, true);

			HTMLHelper::_('easyshop.framework');
			$uri           = JUri::getInstance();
			$root          = $uri->root();
			$pathRoot      = $uri->root(true);
			$base          = $uri->base();
			$pathBase      = $uri->base(true);
			$current       = $uri->toString();
			$currentBase   = base64_encode($current);
			$currencyClass = easyshop('class.currency')->getActive();
			$document      = easyshop('doc');
			$config        = easyshop('config');
			$mediaSets     = [];
			$currency      = [
				'format'    => $currencyClass->get('format'),
				'symbol'    => $currencyClass->get('symbol'),
				'decimals'  => $currencyClass->get('decimals'),
				'separator' => $currencyClass->get('separator'),
				'point'     => $currencyClass->get('point'),
				'code'      => $currencyClass->get('code'),
			];

			if ($config->get('image_lazy_load', '0'))
			{
				$sizeOnScreens = [
					'imagesize_xsmall_screen' => [0, 539, ''],
					'imagesize_small_screen'  => [540, 959, ''],
					'imagesize_medium_screen' => [960, 1199, ''],
					'imagesize_large_screen'  => [1200, 1599, ''],
					'imagesize_xlarge_screen' => [1600, 0, ''],
				];

				foreach ($sizeOnScreens as $name => $mediaCondition)
				{
					if ($size = $config->get($name, ''))
					{
						$mediaCondition[2] = $size;
						$mediaSets[]       = $mediaCondition;
					}
				}
			}

			$document->addScriptDeclaration('			
				_es.setData(\'uri\', {
					\'current\': \'' . $current . '\',
					\'currentBase\': \'' . $currentBase . '\',
					\'base\': \'' . $base . '\',
					\'pathBase\': \'' . $pathBase . '\',
					\'root\': \'' . $root . '\',
					\'pathRoot\': \'' . $pathRoot . '\',
					\'input\': ' . json_encode($this->app->input->getArray()) . '				
				});
				_es.setData(\'currency\', ' . json_encode($currency) . ');
				_es.setData(\'jVersion\', \'' . ES_DETECT_JVERSION . '\');
				_es.setData(\'esVersion\', \'' . ES_VERSION . '\');
				_es.setData(\'token\', \'' . JSession::getFormToken() . '\');
				_es.setData(\'mediaSets\', ' . ($mediaSets ? json_encode($mediaSets) : '[]') . ');
			');

			if ($this->isCategories())
			{
				easyshop('class.html')->initChosen();
			}

			if (easyshop('config', 'user_redirect_form', 0))
			{
				$task = strtolower($this->app->input->getCmd('task', 'display'));
				$view = strtolower($this->app->input->getCmd('view'));

				if ($task === 'display'
					&& in_array($view, ['login', 'registration'])
					&& !CMSFactory::getUser()->id
				)
				{
					require_once ES_COMPONENT_SITE . '/helpers/route.php';
					$this->app->redirect(JRoute::_(EasyshopHelperRoute::getCustomerRoute(), false));
				}
			}

			if ($document->getType() == 'html' && !is_writable(ES_MEDIA))
			{
				$this->app->enqueueMessage(JText::sprintf('COM_EASYSHOP_DIRECTORY_IS_NOT_WRITABLE_FORMAT', str_replace(JPATH_ROOT, '', ES_MEDIA)), 'warning');
			}
		}
		catch (RuntimeException $e)
		{
			$this->app->enqueueMessage($e->getMessage(), 'error');
		}
	}

	public function onAfterDispatch()
	{
		$document = CMSFactory::getDocument();

		if (easyshop('administrator')
			&& $this->isCategories()
			&& $document->getType() == 'html'
		)
		{
			PluginHelper::importPlugin('easyshop');

			if ($this->app->input->get('layout', 'default') == 'default')
			{
				JLoader::import('helpers.navbar', ES_COMPONENT_ADMINISTRATOR);
				$buffer = $document->getBuffer('component');
				$navbar = Navbar::render();
				$dom    = new DOMDocument;
				$string = mb_convert_encoding('<div id="es-component" class="es-category es-scope uk-scope">' . $navbar . '<div id="es-body" class="uk-width-3-4@m uk-width-4-5@xl uk-width-2-3@s">' . $buffer . '</div></div>', 'HTML-ENTITIES', 'UTF-8');

				if (@$dom->loadHTML($string))
				{
					if ($element = $dom->getElementById('es-component'))

						if ($child = $element->ownerDocument->getElementById('j-sidebar-container'))
						{
							$child->parentNode->removeChild($child);
						}

					if ($child = $element->ownerDocument->getElementById('j-main-container'))
					{
						$child->setAttribute('id', 'es-main-container');
						$child->setAttribute('class', 'es-main-container');
					}

					$document->setBuffer($dom->saveHTML($element), 'component');
				}
			}

			$this->app->triggerEvent('onEasyshopCategoryAfterDispatch');
		}
	}

	public function onEasyshopBeforeDispatch()
	{
		$view = $this->app->input->getCmd('view', 'dashboard');

		if ($view == 'dashboard' || $view == 'methods')
		{
			easyshop('doc')->addScriptDeclaration('
				_es.$(document).ready(function($){
					$(\'#es-navbar-content .uk-tab li\').removeClass(\'uk-active\');
					$(\'#es-navbar-content .uk-tab > li:eq(0)\').addClass(\'uk-active\');
				});
			');
		}
	}

	protected function isCategories()
	{
		$option    = $this->app->input->getCmd('option');
		$extension = $this->app->input->getCmd('extension');

		return in_array($option, ['com_categories', 'com_easyshop'])
			&& strpos($extension, 'com_easyshop') !== false;
	}

	public function onContentPrepareForm($form, $data)
	{
		JLoader::register('EasyshopHelper', ES_COMPONENT_ADMINISTRATOR . '/helpers/easyshop.php');
		/** @var $params \ES\Params */
		$params     = easyshop('class.params');
		$extension  = $this->app->input->getCmd('extension');
		$formName   = $form->getName();
		$isCategory = $this->isCategories();
		$registry   = new Registry($data);

		if ($formName == 'com_plugins.plugin')
		{
			$folder = $registry->get('folder');

			if (strpos($folder, 'easyshop') === 0)
			{
				PluginHelper::importPlugin($folder);
			}
		}
		elseif ($isCategory)
		{
			if ($extension == 'com_easyshop.product')
			{
				$form->loadFile(ES_COMPONENT_ADMINISTRATOR . '/models/forms/category/icon.xml');
				$form->load($params->getData('product_listing'));
			}

			PluginHelper::importPlugin('easyshop');
			$this->app->triggerEvent('onEasyshopCategoryPrepareForm', [$form, $data]);
		}

		if ($isCategory
			&& in_array($extension, [
				'com_easyshop.product.customfield',
				'com_easyshop.product.option',
			])
		)
		{
			$form->loadFile(ES_COMPONENT_ADMINISTRATOR . '/models/forms/category/customfield.xml');
		}
		elseif ($formName == 'com_menus.item')
		{
			$option = $registry->get('request.option');
			$view   = $registry->get('request.view');

			if ($option == 'com_easyshop')
			{
				PluginHelper::importPlugin('easyshop');
				$this->app->triggerEvent('onEasyshopMenuPrepareForm', [$form, $data]);

				if ($view == 'productlist')
				{
					$form->load($params->getData('product_listing'));
				}
				elseif ($view == 'productdetail')
				{
					$form->load($params->getData('product_detail'));
				}
			}
		}
		elseif ($formName == 'com_modules.module')
		{
			$jform  = $this->app->input->get('jform', [], 'array');
			$module = isset($jform['module']) ? $jform['module'] : $registry->get('module');

			if (strpos($module, 'mod_easyshop') === 0)
			{
				PluginHelper::importPlugin('easyshop');
			}

			if ($module === 'mod_easyshop_products')
			{
				if ($form->load($params->getData('product_listing', 'basic')))
				{
					foreach (['show_category',
						         'category_description',
						         'show_sub_categories',
						         'product_in_sub_categories',
						         'title_sub_categories'] as $name)
					{
						$form->setFieldAttribute($name, 'showon', 'product_mode:categories', 'params');
					}

					$form->removeField('show_filter_bar', 'params');
					$form->removeField('show_toggle_button', 'params');
				}
			}
		}
	}

	public function onUserAfterDelete($user, $success, $msg)
	{
		if (!$success)
		{
			return false;
		}

		$userId = ArrayHelper::getValue($user, 'id', 0, 'int');
		Table::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/tables');

		$userTable = Table::getInstance('User', 'EasyshopTable');

		if ($userId && $userTable->load(['user_id' => $userId]))
		{
			$userTable->delete($userTable->id);
		}
	}

	public function onUserAfterSave($userData, $isNew, $result, $errorMsg)
	{
		if ($result && $isNew)
		{
			Table::addIncludePath(ES_COMPONENT_ADMINISTRATOR . '/tables');
			$userTable = Table::getInstance('User', 'EasyshopTable');
			$userTable->set('user_id', (int) $userData['id']);
			$userTable->set('state', 1);

			if (!$userTable->check() || !$userTable->store())
			{
				$this->app->enqueueMessage(join("\n", $userTable->getErrors()));

				return false;
			}

			easyshop('state')->set('customer.juser_register_id', $userData['id']);
		}
	}

	public function onContentAfterSave($context, $table, $isNew)
	{
		if ($this->isCategories())
		{
			PluginHelper::importPlugin('easyshop');
			$this->app->triggerEvent('onEasyshopCategoryAfterSave', [$context, $table, $isNew]);
		}
	}

	public function onExtensionAfterSave($context, $table, $isNew)
	{
		if ($context == 'com_plugins.plugin' && in_array($table->folder, ['easyshop', 'easyshoppayment', 'easyshopshipping']))
		{
			PluginHelper::importPlugin($table->folder, $table->element);
			$this->app->triggerEvent('on' . ucfirst($table->folder) . ucfirst($table->element) . 'AfterSave', [$context, $table, $isNew]);
		}
	}

	public function onAfterGetMenuTypeOptions(&$list, $model)
	{
		PluginHelper::importPlugin('easyshop');
		$options = $list['com_easyshop'];
		$this->app->triggerEvent('onEasyshopGetMenuTypeOptions', [&$options, $model]);
		$list['com_easyshop'] = $options;
	}
}

if (!function_exists('easyshop'))
{
	function easyshop($key = null)
	{
		$factory = Factory::getInstance();

		switch ($key)
		{
			case null:
				return $factory;

			case 'app':
			case 'db':
			case 'doc':
			case 'state':
				return $factory->__get($key);

			case 'site':
			case 'administrator':
				return $factory->__get('app')->getName() === $key;

			case 'renderer':
				$renderer = $factory->getClass('LayoutHelper');
				$renderer->refreshDefaultPaths();

				return $renderer;
		}


		$args = func_get_args();
		array_shift($args);

		if (strpos($key, '.') !== false)
		{
			$parts = explode('.', $key, 2);

			if ($parts[0] === 'class')
			{
				array_unshift($args, $parts[1]);

				return call_user_func_array([$factory, 'getClass'], $args);
			}

			if ($parts[0] === 'prepare')
			{
				$data    = $args[0];
				$context = 'com_easyshop' . str_replace([$parts[0], 'com_easyshop.'], '', $key);
				$app     = $factory->__get('app');

				switch (strtolower(gettype($data)))
				{
					case 'array':
						$app->triggerEvent('onEasyshopPrepareItems', [$context, &$data]);
						break;

					case 'object':
						$app->triggerEvent('onEasyshopPrepareItem', [$context, $data]);
						break;

					case 'string':
						$o       = new stdClass;
						$o->text = $data;
						$app->triggerEvent('onEasyshopPrepareContent', [$context, $o]);
						$data = $o->text;
						break;
				}

				return $data;
			}
		}

		$callBack = 'get' . ucfirst($key);

		if (!is_callable([$factory, $callBack]))
		{
			$callBack = $key;
		}

		if (is_callable([$factory, $callBack]))
		{
			return call_user_func_array([$factory, $callBack], $args);
		}
	}

}
